<?php

namespace DevMode\Plugin\MySql\Database;

use NoccyLabs\DevMode\Plugin;
use NoccyLabs\DevMode\Resource;

class MySqlDatabasePlugin extends Plugin implements Plugin\IProvisioner
{
    public function onLoad()
    {
    }

    public function canProvisionResource(Resource $resource)
    {
        if (fnmatch("mysql/*", $resource->getName())) {
            return 1;
        }
        if ($resource->getName() == "mysql") {
            return 1;
        }
    }

    public function provisionResource(Resource $resource, $state)
    {
        $resname = $resource->getName();
        
        $dbname = $resource->name;
        $dbuser = $resource->user;
        $dbpass = $resource->pass;
        $dbhost = $resource->host?:"localhost";
        $dbseed = $resource->seed;

        if (strpos($resname, "/")!==false) {
            list($void,$dbname) = explode("/",$resname,2);
            if (strpos($dbname,"@")!==false) {
                list($dbuser,$dbname) = explode("@",$dbname,2);
            }
        }
        if (!$dbuser) { 
            $dbuser = $dbname;
        }
        if (!$dbpass) {
            $dbpass = $dbname;
        }
        
        $pdo = new \PDO("mysql:host=localhost", "root");
        $pdo->setAttribute(\PDO::ATTR_ERRMODE, \PDO::ERRMODE_EXCEPTION);
        if ($state) {
            if ($this->checkDatabase($pdo, $dbname)) {
                $this->info("Database {$dbname} already created");
                return true;
            }
            $this->createDatabase($pdo, $dbname, $dbuser, $dbpass, $dbhost);
            if ($dbseed) {
                $this->seedDatabase($pdo, $dbname, $dbseed);
            }
        } else {
            if (!$this->checkDatabase($pdo, $dbname)) {
                $this->info("Database {$dbname} does not exist");
                return true;
            }
            $this->dropDatabase($pdo, $dbname);
        }
    }

    private function checkDatabase($pdo, $name)
    {
        $stmt = $pdo->prepare("SHOW DATABASES LIKE :name");
        $stmt->execute(['name'=>$name]);
        $ret = $stmt->fetchAll();
        return (count($ret)>0);
    }

    private function createDatabase($pdo, $name,$user,$pass,$host)
    {
        $this->info("Creating database {$name}");
        $pdo->exec("CREATE DATABASE {$name};");
        $grant = "GRANT ALL ON {$name}.* TO '{$user}'@'{$host}'";
        if ($pass) {
            $grant.=" IDENTIFIED BY '{$pass}'"; 
        }
        $pdo->exec($grant);
    }

    private function seedDatabase($pdo,$name,$file)
    {
        if (!is_file($file)) {
            $this->error("The seed file {$file} could not be found");
            return false;
        }

        $this->info("Seeding database from {$file}");

        if (fnmatch("*.gz",$file)) {
            $fp = fopen("compress.zlib://{$file}", "r");
        } elseif (fnmatch("*.bz2",$file)) {
            $fp = fopen("compress.bzip2://{$file}", "r");
        } else {
            $fp = fopen($file, "r");
        }
        if (!$fp) {
            $this->error("Could not open {$file} (tried zlib, bzip2, plain)");
            return false;
        }

        $pdo->exec("USE {$name}");
        while (!feof($fp)) {
            $stmt = fgets($fp);
            $pdo->exec($stmt);
        }

        return true;
    }

    private function dropDatabase($pdo,$name)
    {
        $this->info("Dropping database {$name}");
        $pdo->exec("DROP DATABASE {$name}");
    }

}

return new MySqlDatabasePlugin();
