<?php

namespace DevMode\Plugin\Symfony\Httpd;

use NoccyLabs\DevMode\Plugin;
use NoccyLabs\DevMode\Dependency;

class SymfonyHttpdPlugin extends Plugin implements Plugin\IDependencyManager
{
    protected $symfony;

    public function onLoad()
    {
        if (is_file("app/console")) {
            $this->symfony = "app/console";
        } elseif (is_file("bin/console")) {
            $this->symfony = "bin/console";
        }
        return true;
    }

    public function isManagedDependency(Dependency $dependency)
    {
        if ($dependency->getName() == "symfony/httpd") {
            return 1;
        }
    }

    public function setDependencyState(Dependency $dependency, $state)
    {
        if (!$this->symfony) {
            throw new \RuntimeException("Could not find the symfony console executable for symfony/httpd");
        }
        $args = [];
        if ($dependency->bind) {
            $args[] = $dependency->bind;
        }
        if ($state == true) {
            $this->execBackground($this->symfony, "server:start", ...$args);
        } else {
            $this->execBackground($this->symfony, "server:stop");
        }
    }

    public function showDependencyStatus(Dependency $dependency)
    {
        $this->exec($this->symfony, "server:status");
    }

}

return new SymfonyHttpdPlugin();
