<?php

namespace NoccyLabs\DevMode;

use Psr\Log\LoggerTrait;
use Psr\Log\LoggerInterface;

abstract class Plugin implements LoggerInterface
{
    use LoggerTrait;

    protected $manifest;
    
    protected $application;

    abstract public function onLoad();
    
    public function setManifest($manifest)
    {
        $this->manifest = $manifest;
    }

    public function getManifest()
    {
        return $this->manifest;
    }
    
    public function setApplication(Application $application)
    {
        $this->application = $application;
    }

    public function getId()
    {
        return $this->manifest->getId();
    }

    protected function exec(...$cmdl)
    {
        $cmd = [ escapeshellcmd(array_shift($cmdl)) ];
        foreach ($cmdl as $arg) {
            $cmd[] = escapeshellarg($arg);
        }
        $cmd = join(" ",$cmd);
        printf("\e[32;1m[%s] exec: %s\e[0m\n", $this->getId(), $cmd);
        exec($cmd, $output, $retval);
        echo "\e[36;96m";
        foreach ($output as $line) {
            if (trim($line)) echo " ░░▌ {$line}\n";
        }
        echo "\e[0m";
    }
    
    protected function execBackground(...$cmdl)
    {
        $cmd = [ escapeshellcmd(array_shift($cmdl)) ];
        foreach ($cmdl as $arg) {
            $cmd[] = escapeshellarg($arg);
        }
        $cmd = join(" ",$cmd);
        printf("\e[32;1m[%s] exec: %s\e[0m\n", $this->getId(), $cmd);
        $pid = pcntl_fork();
        if ($pid === 0) {
            exec($cmd); // "sh -c \"{$cmd}\" &>/dev/null &");
            exit(0);
        }
    }
    
    protected function execSudo(...$cmdl)
    {
        // note: gksudo doesn't remember auth like sudo does, prompts repeatedly...
        //if (getenv('DISPLAY') && (trim(exec("which gksudo")))) {
        //    $this->exec("gksudo", "-kS", ...$cmdl);
        //} else {
            $this->exec("sudo", ...$cmdl);
        //}
    }

    public function log($level, $text, array $context=[])
    {
        switch ($level) {
            case "debug": 
                echo "\e[32;92m"; break;
            case "info":
                echo "\e[34;94m"; break;
            case "warning":
            case "error":
                echo "\e[31;91m"; break;
        }
        printf("[%s] (%s) %s\e[0m\n", $this->getId(), $level, $text);
    }
}
